@@ -28,7 +28,6 @@ import contextlib
 import logging
 from typing import Optional, Union
 
-from pytype import special_builtins
 from pytype.abstract import abstract
 from pytype.abstract import abstract_utils
 from pytype.abstract import class_mixin
@@ -36,6 +35,7 @@ from pytype.abstract import function
 from pytype.overlays import classgen
 from pytype.overlays import overlay
 from pytype.overlays import overlay_utils
+from pytype.overlays import special_builtins
 from pytype.pytd import pytd
 from pytype.pytd import pytd_utils
 from pytype.pytd import visitors
@@ -241,6 +241,14 @@ class EnumInstance(abstract.InterpreterClass):
           return False
     return True
 
+  def get_enum_members(self, qualified=False):
+    ret = {k: v for k, v in self.members.items()
+           if all(d.cls == self for d in v.data)}
+    if qualified:
+      return {f"{self.name}.{k}": v for k, v in ret.items()}
+    else:
+      return ret
+
 
 class EnumCmpEQ(abstract.SimpleFunction):
   """Implements the functionality of __eq__ for an enum."""
