@@ -128,6 +128,54 @@ PYTHON_MAGIC = {
     3437: (3, 10),  # b1
     3438: (3, 10),  # b1
     3439: (3, 10),  # b1
+
+    # Python 3.11
+    3450: (3, 11),  # a1
+    3451: (3, 11),  # a1
+    3452: (3, 11),  # a1
+    3453: (3, 11),  # a1
+    3454: (3, 11),  # a1
+    3455: (3, 11),  # a1
+    3456: (3, 11),  # a1
+    3457: (3, 11),  # a1
+    3458: (3, 11),  # a1
+    3459: (3, 11),  # a1
+    3460: (3, 11),  # a1
+    3461: (3, 11),  # a1
+    3462: (3, 11),  # a2
+    3463: (3, 11),  # a3
+    3464: (3, 11),  # a3
+    3465: (3, 11),  # a3
+    3466: (3, 11),  # a4
+    3467: (3, 11),  # a4
+    3468: (3, 11),  # a4
+    3469: (3, 11),  # a4
+    3470: (3, 11),  # a4
+    3471: (3, 11),  # a4
+    3472: (3, 11),  # a4
+    3473: (3, 11),  # a4
+    3474: (3, 11),  # a4
+    3475: (3, 11),  # a5
+    3476: (3, 11),  # a5
+    3477: (3, 11),  # a5
+    3478: (3, 11),  # a5
+    3479: (3, 11),  # a5
+    3480: (3, 11),  # a5
+    3481: (3, 11),  # a5
+    3482: (3, 11),  # a5
+    3483: (3, 11),  # a5
+    3484: (3, 11),  # a5
+    3485: (3, 11),  # a5
+    3486: (3, 11),  # a6
+    3487: (3, 11),  # a6
+    3488: (3, 11),  # a6
+    3489: (3, 11),  # a6
+    3490: (3, 11),  # a6
+    3491: (3, 11),  # a6
+    3492: (3, 11),  # a7
+    3493: (3, 11),  # a7
+    3494: (3, 11),  # a7
+    3495: (3, 11),  # b4
 }
 
 
