@@ -431,8 +431,10 @@ def _expand_generic_protocols(node, bases, ctx):
 
 def _check_final_members(cls, class_dict, ctx):
   """Check if the new class overrides a final attribute or method."""
-  methods = class_dict.keys()
+  methods = set(class_dict)
   for base in cls.mro[1:]:
+    if not isinstance(base, abstract.Class):
+      continue
     if isinstance(base, abstract.PyTDClass):
       # TODO(mdemello): Unify this with IntepreterClass
       for m in methods:
@@ -441,7 +443,7 @@ def _check_final_members(cls, class_dict, ctx):
           ctx.errorlog.overriding_final_method(ctx.vm.frames, cls, base, m)
         elif member:
           ctx.errorlog.overriding_final_attribute(ctx.vm.frames, cls, base, m)
-    elif isinstance(base, abstract.Class):
+    else:
       for m in methods:
         if m in base.members:
           if any(x.final for x in base.members[m].data):
@@ -449,6 +451,7 @@ def _check_final_members(cls, class_dict, ctx):
           ann = base.get_annotated_local(m)
           if ann and ann.final:
             ctx.errorlog.overriding_final_attribute(ctx.vm.frames, cls, base, m)
+    methods.update(base.get_own_attributes())
 
 
 def make_class(node, props, ctx):
@@ -1101,9 +1104,7 @@ def copy_dict_without_keys(
   # NOTE: This handles the specific case of a dict with a concrete pyval, where
   # we can track keys and values. Otherwise just return the object with type
   # unchanged; at worst it will be a superset of the correct type.
-  if not all(isinstance(x, abstract.Dict) for x in obj_var.data):
-    return obj_var
-  if any(x.could_contain_anything for x in obj_var.data):
+  if not all(abstract_utils.is_concrete_dict(x) for x in obj_var.data):
     return obj_var
   keys = _convert_keys(keys_var)
   ret = abstract.Dict(ctx)
